@@ -5,17 +5,19 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from typing import Optional, Dict, Awaitable, List, Tuple, Union, TYPE_CHECKING
 from urllib.parse import quote as urllib_quote
+import asyncio
 
-from ...api import Method, Path
-from ...types import (StateEvent, EventType, StateEventContent, EventID, ContentURI,
-                      MessageEventContent, UserID, RoomID, PresenceState,
-                      RoomAvatarStateEventContent, RoomNameStateEventContent,
-                      RoomTopicStateEventContent, PowerLevelStateEventContent,
-                      RoomPinnedEventsStateEventContent, Membership, Member,
-                      MemberStateEventContent)
-from ...client import ClientAPI
-from ...errors import MForbidden, MBadState, MatrixRequestError, IntentError
-from ..state_store import StateStore
+from mautrix.api import Method, Path
+from mautrix.types import (StateEvent, EventType, StateEventContent, EventID, ContentURI,
+                           EventContent, UserID, RoomID, PresenceState,
+                           RoomAvatarStateEventContent, RoomNameStateEventContent,
+                           RoomTopicStateEventContent, PowerLevelStateEventContent,
+                           RoomPinnedEventsStateEventContent, Membership, Member)
+from mautrix.client import ClientAPI, StoreUpdatingAPI
+from mautrix.errors import MForbidden, MBadState, MatrixRequestError, IntentError
+from mautrix.util.logging import TraceLogger
+
+from ..state_store import ASStateStore
 
 try:
     import magic
@@ -52,20 +54,22 @@ ENSURE_JOINED_METHODS = (
 )
 
 
-class IntentAPI(ClientAPI):
+class IntentAPI(StoreUpdatingAPI):
     """
     IntentAPI is a high-level wrapper around the AppServiceAPI that provides many easy-to-use
     functions for accessing the client-server API. It is designed for appservices and will
     automatically handle many things like missing invites using the appservice bot.
     """
     api: 'AppServiceAPI'
+    state_store: ASStateStore
+    bot: 'IntentAPI'
+    log: TraceLogger
 
     def __init__(self, mxid: UserID, api: 'AppServiceAPI', bot: 'IntentAPI' = None,
-                 state_store: StateStore = None):
-        super().__init__(mxid, api)
+                 state_store: ASStateStore = None):
+        super().__init__(mxid=mxid, api=api, state_store=state_store)
         self.bot = bot
         self.log = api.base_log.getChild("intent")
-        self.state_store = state_store
 
         for method in ENSURE_REGISTERED_METHODS:
             method = getattr(self, method.__name__)
@@ -154,16 +158,16 @@ class IntentAPI(ClientAPI):
         """
         try:
             ok_states = (Membership.INVITE, Membership.JOIN)
-            do_invite = (not check_cache
-                         or self.state_store.get_membership(room_id, user_id) not in ok_states)
+            do_invite = (not check_cache or (await self.state_store.get_membership(room_id, user_id)
+                                             not in ok_states))
             if do_invite:
                 await super().invite_user(room_id, user_id)
-                self.state_store.invited(room_id, user_id)
+                await self.state_store.invited(room_id, user_id)
         except MatrixRequestError as e:
             if e.errcode != "M_FORBIDDEN":
                 raise IntentError(f"Failed to invite {user_id} to {room_id}", e)
             if "is already in the room" in e.message:
-                self.state_store.joined(room_id, user_id)
+                await self.state_store.joined(room_id, user_id)
 
     def set_room_avatar(self, room_id: RoomID, avatar_url: Optional[ContentURI], **kwargs
                         ) -> Awaitable[EventID]:
@@ -183,14 +187,11 @@ class IntentAPI(ClientAPI):
                                ) -> PowerLevelStateEventContent:
         await self.ensure_joined(room_id)
         if not ignore_cache:
-            try:
-                levels = self.state_store.get_power_levels(room_id)
-            except KeyError:
-                levels = None
+            levels = await self.state_store.get_power_levels(room_id)
             if levels:
                 return levels
         levels = await self.get_state_event(room_id, EventType.ROOM_POWER_LEVELS)
-        self.state_store.set_power_levels(room_id, levels)
+        await self.state_store.set_power_levels(room_id, levels)
         return levels
 
     async def set_power_levels(self, room_id: RoomID, content: PowerLevelStateEventContent, **kwargs
@@ -198,7 +199,7 @@ class IntentAPI(ClientAPI):
         response = await self.send_state_event(room_id, EventType.ROOM_POWER_LEVELS, content,
                                                **kwargs)
         if response:
-            self.state_store.set_power_levels(room_id, content)
+            await self.state_store.set_power_levels(room_id, content)
         return response
 
     async def get_pinned_messages(self, room_id: RoomID) -> List[EventID]:
@@ -206,8 +207,8 @@ class IntentAPI(ClientAPI):
         content = await self.get_state_event(room_id, EventType.ROOM_PINNED_EVENTS)
         return content["pinned"]
 
-    def set_pinned_messages(self, room_id: RoomID, events: List[EventID], **kwargs) -> Awaitable[
-        dict]:
+    def set_pinned_messages(self, room_id: RoomID, events: List[EventID], **kwargs
+                            ) -> Awaitable[EventID]:
         return self.send_state_event(room_id, EventType.ROOM_PINNED_EVENTS,
                                      RoomPinnedEventsStateEventContent(pinned=events), **kwargs)
 
@@ -240,8 +241,8 @@ class IntentAPI(ClientAPI):
 
     async def get_room_member_info(self, room_id: RoomID, user_id: UserID, ignore_cache=False
                                    ) -> Member:
-        member = self.state_store.get_member(room_id, user_id)
-        if not member.membership or ignore_cache:
+        member = await self.state_store.get_member(room_id, user_id)
+        if not member or not member.membership or ignore_cache:
             member = await self.get_state_event(room_id, EventType.ROOM_MEMBER, user_id)
         return member
 
@@ -259,28 +260,8 @@ class IntentAPI(ClientAPI):
         await self.send_notice(room_id, text, html=html)
         await self.leave_room(room_id)
 
-    def get_membership(self, room_id: RoomID, user_id: UserID
-                       ) -> Awaitable[MemberStateEventContent]:
-        return self.get_state_event(room_id, EventType.ROOM_MEMBER, state_key=user_id)
-
-    def set_membership(self, room_id: RoomID, user_id: UserID, membership: Membership,
-                       reason: Optional[str] = "", profile: Optional[dict] = None, **kwargs
-                       ) -> Awaitable[dict]:
-        body = {
-            "membership": membership.serialize(),
-            "reason": reason
-        }
-        profile = profile or {}
-        if "displayname" in profile:
-            body["displayname"] = profile["displayname"]
-        if "avatar_url" in profile:
-            body["avatar_url"] = profile["avatar_url"]
-
-        return self.send_state_event(room_id, EventType.ROOM_MEMBER, body, state_key=user_id,
-                                     **kwargs)
-
     async def send_message_event(self, room_id: RoomID, event_type: EventType,
-                                 content: Union[MessageEventContent, Dict], **kwargs) -> EventID:
+                                 content: EventContent, **kwargs) -> EventID:
         await self._ensure_has_power_level_for(room_id, event_type)
 
         if self.api.is_real_user and self.api.real_user_content_key:
@@ -294,49 +275,30 @@ class IntentAPI(ClientAPI):
         await self._ensure_has_power_level_for(room_id, event_type)
         return await super().send_state_event(room_id, event_type, content, state_key, **kwargs)
 
-    async def get_state_event(self, room_id: RoomID, event_type: EventType,
-                              state_key: Optional[str] = "") -> StateEventContent:
-        event = await super().get_state_event(room_id, event_type, state_key)
-        self.state_store.update_state(StateEvent(type=event_type, room_id=room_id,
-                                                 event_id=EventID(""), sender=UserID(""),
-                                                 state_key=state_key, timestamp=0, content=event))
-        return event
+    async def get_room_members(self, room_id: RoomID,
+                               allowed_memberships: Tuple[Membership, ...] = (Membership.JOIN,)
+                               ) -> List[UserID]:
+        if len(allowed_memberships) == 1 and allowed_memberships[0] == Membership.JOIN:
+            memberships = await self.get_joined_members(room_id)
+            return list(memberships.keys())
+        member_events = await self.get_members(room_id)
+        return [evt.state_key for evt in member_events
+                if evt.content.membership in allowed_memberships]
 
     async def leave_room(self, room_id: RoomID) -> None:
         if not room_id:
             raise ValueError("Room ID not given")
         await self.ensure_registered()
         try:
-            self.state_store.left(room_id, self.mxid)
+            await self.state_store.left(room_id, self.mxid)
             await super().leave_room(room_id)
         except MatrixRequestError as e:
             if "not in room" not in e.message:
                 raise
 
-    def get_room_memberships(self, room_id: RoomID) -> Awaitable[dict]:
-        if not room_id:
-            raise ValueError("Room ID not given")
-        return self.api.request(Method.GET, Path.rooms[room_id].members)
-
-    def get_room_joined_memberships(self, room_id: RoomID) -> Awaitable[dict]:
-        if not room_id:
-            raise ValueError("Room ID not given")
-        return self.api.request(Method.GET, Path.rooms[room_id].joined_members)
-
-    async def get_room_members(self, room_id: RoomID,
-                               allowed_memberships: Tuple[Membership, ...] = (Membership.JOIN,)
-                               ) -> List[UserID]:
-        if len(allowed_memberships) == 1 and allowed_memberships[0] == Membership.JOIN:
-            memberships = await self.get_room_joined_memberships(room_id)
-            return list(memberships["joined"].keys())
-        memberships = await self.get_room_memberships(room_id)
-        return [membership["state_key"] for membership in memberships["chunk"] if
-                Membership(membership["content"]["membership"]) in allowed_memberships]
-
     async def get_state(self, room_id: RoomID) -> List[StateEvent]:
         state = await super().get_state(room_id)
-        for event in state:
-            self.state_store.update_state(event)
+        await asyncio.gather(*[self.state_store.update_state(evt) for evt in state])
         return state
 
     async def mark_read(self, room_id: RoomID, event_id: EventID) -> None:
@@ -347,23 +309,22 @@ class IntentAPI(ClientAPI):
     # endregion
     # region Ensure functions
 
-    async def ensure_joined(self, room_id: RoomID, ignore_cache: bool = False) -> None:
+    async def ensure_joined(self, room_id: RoomID, ignore_cache: bool = False) -> bool:
         if not room_id:
             raise ValueError("Room ID not given")
-        if not ignore_cache and self.state_store.is_joined(room_id, self.mxid):
-            return
+        if not ignore_cache and await self.state_store.is_joined(room_id, self.mxid):
+            return False
         await self.ensure_registered()
         try:
             await self.join_room(room_id, max_retries=0)
-            self.state_store.joined(room_id, self.mxid)
+            await self.state_store.joined(room_id, self.mxid)
         except MForbidden as e:
             if not self.bot:
                 raise IntentError(f"Failed to join room {room_id} as {self.mxid}") from e
             try:
                 await self.bot.invite_user(room_id, self.mxid)
                 await self.join_room(room_id, max_retries=0)
-                self.state_store.joined(room_id, self.mxid)
-                return
+                await self.state_store.joined(room_id, self.mxid)
             except MatrixRequestError as e2:
                 raise IntentError(f"Failed to join room {room_id} as {self.mxid}") from e2
         except MBadState as e:
@@ -373,12 +334,12 @@ class IntentAPI(ClientAPI):
                 await self.bot.unban_user(room_id, self.mxid)
                 await self.bot.invite_user(room_id, self.mxid)
                 await self.join_room(room_id, max_retries=0)
-                self.state_store.joined(room_id, self.mxid)
-                return
+                await self.state_store.joined(room_id, self.mxid)
             except MatrixRequestError as e2:
                 raise IntentError(f"Failed to join room {room_id} as {self.mxid}") from e2
         except MatrixRequestError as e:
             raise IntentError(f"Failed to join room {room_id} as {self.mxid}") from e
+        return True
 
     def _register(self) -> Awaitable[dict]:
         content = {"username": self.localpart, "type": "m.login.application_service"}
@@ -386,7 +347,7 @@ class IntentAPI(ClientAPI):
         return self.api.request(Method.POST, Path.register, content, query_params=query_params)
 
     async def ensure_registered(self) -> None:
-        if self.state_store.is_registered(self.mxid):
+        if await self.state_store.is_registered(self.mxid):
             return
         try:
             await self._register()
@@ -395,7 +356,7 @@ class IntentAPI(ClientAPI):
                 raise IntentError(f"Failed to register {self.mxid}", e)
                 # self.log.exception(f"Failed to register {self.mxid}!")
                 # return
-        self.state_store.registered(self.mxid)
+        await self.state_store.registered(self.mxid)
 
     async def _ensure_has_power_level_for(self, room_id: RoomID, event_type: EventType) -> None:
         if not room_id:
@@ -403,9 +364,9 @@ class IntentAPI(ClientAPI):
         elif not event_type:
             raise ValueError("Event type not given")
 
-        if not self.state_store.has_power_levels(room_id):
-            await self.get_power_levels(room_id)
-        if not self.state_store.has_power_level(room_id, self.mxid, event_type):
+        if not await self.state_store.has_power_levels_cached(room_id):
+            await self.get_power_levels(room_id, ignore_cache=True)
+        if not await self.state_store.has_power_level(room_id, self.mxid, event_type):
             # TODO implement something better
             raise IntentError(f"Power level of {self.mxid} is not enough "
                               f"for {event_type} in {room_id}")
