@@ -19,6 +19,7 @@ if TYPE_CHECKING:
     from ..bridge import Bridge
 
 command_handlers: Dict[str, 'CommandHandler'] = {}
+command_aliases: Dict[str, 'CommandHandler'] = {}
 
 HelpSection = NamedTuple('HelpSection', name=str, order=int, description=str)
 HelpCacheKey = NamedTuple('HelpCacheKey', is_management=bool, is_portal=bool)
@@ -26,7 +27,6 @@ HelpCacheKey = NamedTuple('HelpCacheKey', is_management=bool, is_portal=bool)
 SECTION_GENERAL = HelpSection("General", 0, "")
 
 
-
 def ensure_trailing_newline(s: str) -> str:
     """Returns the passed string, but with a guaranteed trailing newline."""
     return s + ("" if s[-1] == "\n" else "\n")
@@ -265,7 +265,7 @@ class CommandHandler:
 
 def command_handler(_func: Optional[CommandHandlerFunc] = None, *, management_only: bool = False,
                     name: Optional[str] = None, help_text: str = "", help_args: str = "",
-                    help_section: HelpSection = None,
+                    help_section: HelpSection = None, aliases: Optional[List[str]] = None,
                     _handler_class: Type[CommandHandler] = CommandHandler,
                     **kwargs) -> Callable[[CommandHandlerFunc], CommandHandler]:
     """Decorator to create CommandHandlers"""
@@ -275,6 +275,9 @@ def command_handler(_func: Optional[CommandHandlerFunc] = None, *, management_on
         handler = _handler_class(func, management_only, actual_name, help_text, help_args,
                                  help_section, **kwargs)
         command_handlers[handler.name] = handler
+        if aliases:
+            for alias in aliases:
+                command_aliases[alias] = handler
         return handler
 
     return decorator if _func is None else decorator(_func)
@@ -291,12 +294,10 @@ class CommandProcessor:
     bridge: 'Bridge'
     _ref_no: int
 
-    def __init__(self, az: AppService, config: BaseBridgeConfig, bridge: 'Bridge',
-                 event_class: Type[CommandEvent] = CommandEvent,
-                 loop: asyncio.AbstractEventLoop = None) -> None:
-        self.az = az
-        self.config = config
-        self.loop = loop or asyncio.get_event_loop()
+    def __init__(self, bridge: 'Bridge', event_class: Type[CommandEvent] = CommandEvent) -> None:
+        self.az = bridge.az
+        self.config = bridge.config
+        self.loop = bridge.loop or asyncio.get_event_loop()
         self.command_prefix = self.config["bridge.command_prefix"]
         self.bridge = bridge
         self.event_class = event_class
@@ -349,12 +350,15 @@ class CommandProcessor:
         try:
             handler = command_handlers[command]
         except KeyError:
-            if sender.command_status and "next" in sender.command_status:
-                args.insert(0, orig_command)
-                evt.command = ""
-                handler = sender.command_status["next"]
-            else:
-                handler = command_handlers["unknown-command"]
+            try:
+                handler = command_aliases[command]
+            except KeyError:
+                if sender.command_status and "next" in sender.command_status:
+                    args.insert(0, orig_command)
+                    evt.command = ""
+                    handler = sender.command_status["next"]
+                else:
+                    handler = command_handlers["unknown-command"]
         try:
             await self._run_handler(handler, evt)
         except Exception:
