@@ -3,12 +3,14 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Tuple, Optional, Union, List, TYPE_CHECKING
-from abc import ABC, abstractmethod
+from typing import Tuple, Optional, Union, TYPE_CHECKING
 import logging
 import asyncio
+import os.path
 import time
 
+from yarl import URL
+
 from mautrix.types import (EventID, RoomID, UserID, Event, EventType, MessageEvent, MessageType,
                            MessageEventContent, StateEvent, Membership, MemberStateEventContent,
                            PresenceEvent, TypingEvent, ReceiptEvent, TextMessageEventContent,
@@ -32,7 +34,7 @@ except ImportError:
     EncryptionManager = None
 
 
-class BaseMatrixHandler(ABC):
+class BaseMatrixHandler:
     log: TraceLogger = logging.getLogger("mau.mx")
     az: AppService
     commands: CommandProcessor
@@ -43,14 +45,12 @@ class BaseMatrixHandler(ABC):
     user_id_prefix: str
     user_id_suffix: str
 
-    def __init__(self, az: AppService, config: 'BaseBridgeConfig',
-                 command_processor: Optional[CommandProcessor] = None,
-                 loop: Optional[asyncio.AbstractEventLoop] = None,
+    def __init__(self, command_processor: Optional[CommandProcessor] = None,
                  bridge: Optional['Bridge'] = None) -> None:
-        self.az = az
-        self.config = config
-        self.commands = command_processor or CommandProcessor(az=az, config=config, loop=loop,
-                                                              bridge=bridge)
+        self.az = bridge.az
+        self.config = bridge.config
+        self.bridge = bridge
+        self.commands = command_processor or CommandProcessor(bridge=bridge)
         self.az.matrix_event_handler(self.int_handle_event)
 
         self.e2ee = None
@@ -61,10 +61,24 @@ class BaseMatrixHandler(ABC):
                 self.log.warning("Encryption enabled in config, but login_shared_secret not set.")
             else:
                 self.e2ee = EncryptionManager(
-                    bot_mxid=self.az.bot_mxid, device_name=bridge.name,
+                    bridge=bridge,
                     user_id_prefix=self.user_id_prefix, user_id_suffix=self.user_id_suffix,
                     login_shared_secret=self.config["bridge.login_shared_secret"],
-                    homeserver_address=self.config["homeserver.address"], loop=loop)
+                    homeserver_address=self.config["homeserver.address"],
+                    db_url=self._get_db_url(bridge.config),
+                    key_sharing_config=self.config["bridge.encryption.key_sharing"])
+
+    @staticmethod
+    def _get_db_url(config: 'BaseBridgeConfig') -> str:
+        db_url = config["bridge.encryption.database"]
+        if not db_url or db_url == "default":
+            db_url = config["appservice.database"]
+            parsed_url = URL(db_url)
+            if parsed_url.scheme == "sqlite":
+                # Remove the extension to replace it with our own
+                path = os.path.splitext(parsed_url.path)[0]
+                db_url = f"pickle://{path}.crypto.pickle"
+        return db_url
 
     async def wait_for_connection(self) -> None:
         self.log.info("Ensuring connectivity to homeserver")
@@ -111,18 +125,6 @@ class BaseMatrixHandler(ABC):
         if self.e2ee:
             await self.e2ee.start()
 
-    @abstractmethod
-    async def get_user(self, user_id: UserID) -> 'BaseUser':
-        pass
-
-    @abstractmethod
-    async def get_portal(self, room_id: RoomID) -> 'BasePortal':
-        pass
-
-    @abstractmethod
-    async def get_puppet(self, user_id: UserID) -> 'BasePuppet':
-        pass
-
     @staticmethod
     async def allow_message(user: 'BaseUser') -> bool:
         return user.is_whitelisted
@@ -229,7 +231,7 @@ class BaseMatrixHandler(ABC):
     async def int_handle_invite(self, room_id: RoomID, user_id: UserID, invited_by: UserID,
                                 event_id: EventID) -> None:
         self.log.debug(f"{invited_by} invited {user_id} to {room_id}")
-        inviter = await self.get_user(invited_by)
+        inviter = await self.bridge.get_user(invited_by)
         if inviter is None:
             self.log.exception(f"Failed to find user with Matrix ID {invited_by}")
             return
@@ -239,7 +241,7 @@ class BaseMatrixHandler(ABC):
         elif not await self.allow_command(inviter):
             return
 
-        puppet = await self.get_puppet(user_id)
+        puppet = await self.bridge.get_puppet(user_id)
         if puppet:
             await self.handle_puppet_invite(room_id, puppet, inviter, event_id)
             return
@@ -256,7 +258,7 @@ class BaseMatrixHandler(ABC):
 
     async def handle_message(self, room_id: RoomID, user_id: UserID, message: MessageEventContent,
                              event_id: EventID) -> None:
-        sender = await self.get_user(user_id)
+        sender = await self.bridge.get_user(user_id)
         if not sender or not await self.allow_message(sender):
             self.log.debug(f"Ignoring message {event_id} from {user_id} to {room_id}:"
                            " User is not whitelisted.")
@@ -268,7 +270,7 @@ class BaseMatrixHandler(ABC):
             message.trim_reply_fallback()
 
         is_command, text = self.is_command(message)
-        portal = await self.get_portal(room_id)
+        portal = await self.bridge.get_portal(room_id)
         if not is_command and portal and await self.allow_bridging_message(sender, portal):
             await portal.handle_matrix_message(sender, message, event_id)
             return
@@ -311,27 +313,18 @@ class BaseMatrixHandler(ABC):
             self.log.exception("Error handling manually received Matrix event")
 
     async def handle_encrypted(self, evt: EncryptedEvent) -> None:
-        await self.int_handle_event(self.e2ee.decrypt(evt))
+        await self.int_handle_event(await self.e2ee.decrypt(evt))
 
-    async def enable_dm_encryption(self, portal: 'BasePortal', members: List[UserID]) -> bool:
-        try:
-            await portal.main_intent.invite_user(portal.mxid, self.az.bot_mxid)
-            await self.az.intent.join_room_by_id(portal.mxid)
-            await portal.main_intent.send_state_event(portal.mxid, EventType.ROOM_ENCRYPTION, {
-                "algorithm": "m.megolm.v1.aes-sha2"
-            })
-        except Exception:
-            self.log.warning(f"Failed to enable end-to-bridge encryption in {portal.mxid}",
-                             exc_info=True)
-            return False
-
-        portal.encrypted = True
-        await self.e2ee.add_room(portal.mxid, members=members + [self.az.bot_mxid], encrypted=True)
-        return True
+    async def handle_encryption(self, evt: StateEvent) -> None:
+        await self.az.state_store.set_encryption_info(evt.room_id, evt.content)
+        portal = await self.bridge.get_portal(evt.room_id)
+        if portal.encrypted:
+            portal.encrypted = True
+            await portal.save()
 
     async def int_handle_event(self, evt: Event) -> None:
         if isinstance(evt, StateEvent) and evt.type == EventType.ROOM_MEMBER and self.e2ee:
-            await self.e2ee.handle_room_membership(evt)
+            await self.e2ee.handle_member_event(evt)
         if self.filter_matrix_event(evt):
             return
         self.log.trace("Received event: %s", evt)
@@ -374,10 +367,10 @@ class BaseMatrixHandler(ABC):
             if evt.type != EventType.ROOM_MESSAGE:
                 evt.content.msgtype = MessageType(str(evt.type))
             await self.handle_message(evt.room_id, evt.sender, evt.content, evt.event_id)
-        elif evt.type == EventType.ROOM_ENCRYPTION and self.e2ee:
-            await self.e2ee.handle_room_encryption(evt)
         elif evt.type == EventType.ROOM_ENCRYPTED and self.e2ee:
             await self.handle_encrypted(evt)
+        elif evt.type == EventType.ROOM_ENCRYPTION:
+            await self.handle_encryption(evt)
         else:
             if evt.type.is_state and isinstance(evt, StateEvent):
                 await self.handle_state_event(evt)
