@@ -6,10 +6,10 @@
 from typing import Optional, Tuple
 import warnings
 import asyncio
-import logging
 
-from ...api import HTTPAPI
-from .types import UserID
+from mautrix.api import HTTPAPI
+from mautrix.types import UserID, DeviceID
+from mautrix.util.logging import TraceLogger
 
 
 class BaseClientAPI:
@@ -22,12 +22,13 @@ class BaseClientAPI:
 
     localpart: str
     domain: str
-    mxid: UserID
+    _mxid: UserID
+    device_id: DeviceID
     api: HTTPAPI
     loop: asyncio.AbstractEventLoop
-    log: logging.Logger
+    log: TraceLogger
 
-    def __init__(self, mxid: UserID, api: HTTPAPI = None,
+    def __init__(self, mxid: UserID = "", device_id: DeviceID = "", api: HTTPAPI = None,
                  loop: Optional[asyncio.AbstractEventLoop] = None, *args, **kwargs) -> None:
         """
         Initialize a ClientAPI. You must either provide the
@@ -39,7 +40,13 @@ class BaseClientAPI:
             api: The :class:`HTTPAPI` instance to use. You can also pass the ``args`` and ``kwargs``
                 to create a HTTPAPI instance rather than creating the instance yourself.``
         """
-        self.set_mxid(mxid)
+        if mxid:
+            self.mxid = mxid
+        else:
+            self._mxid = None
+            self.localpart = None
+            self.domain = None
+        self.device_id = device_id
         if loop:
             kwargs["loop"] = loop
         self.api = api or HTTPAPI(*args, **kwargs)
@@ -78,12 +85,11 @@ class BaseClientAPI:
             raise ValueError("User ID must contain domain")
         return mxid[1:sep], mxid[sep + 1:]
 
-    def set_mxid(self, mxid: UserID) -> None:
-        """
-        Update the Matrix user ID used by this ClientAPI.
+    @property
+    def mxid(self) -> UserID:
+        return self._mxid
 
-        Args:
-            mxid: The new Matrix user ID.
-        """
+    @mxid.setter
+    def mxid(self, mxid: UserID) -> None:
         self.localpart, self.domain = self.parse_user_id(mxid)
-        self.mxid = mxid
+        self._mxid = mxid
