@@ -5,13 +5,15 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from .media_repository import MediaRepositoryMethods
 from .misc import MiscModuleMethods
+from .crypto import CryptoMethods
+from .account_data import AccountDataMethods
 
 
-class ModuleMethods(MediaRepositoryMethods, MiscModuleMethods):
+class ModuleMethods(MediaRepositoryMethods, CryptoMethods, AccountDataMethods, MiscModuleMethods):
     """
     Methods in section 13 Modules of the spec.
 
-    See also: `API reference <https://matrix.org/docs/spec/client_server/r0.4.0.html#modules>`__
+    See also: `API reference <https://matrix.org/docs/spec/client_server/r0.6.1.html#modules>`__
     """
 
-    # TODO: subregions 15, 18, 19, 21, 26, 27, others?
+    # TODO: subregions 15, 18, 21, 26, 27, others?
