@@ -6,4 +6,7 @@ from .request import (MatrixRequestError, MatrixUnknownRequestError, MatrixStand
                       MLimitExceeded, MUnknown, MUnrecognized, MUnauthorized, MUserInUse,
                       MInvalidUsername, MRoomInUse, MInvalidRoomState, MUnsupportedRoomVersion,
                       MIncompatibleRoomVersion, MBadState, MGuestAccessForbidden, MCaptchaNeeded,
-                      MCaptchaInvalid, MMissingParam, MInvalidParam, MTooLarge, MExclusive)
+                      MCaptchaInvalid, MMissingParam, MInvalidParam, MTooLarge, MExclusive,
+                      MUserDeactivated)
+from .crypto import (CryptoError, EncryptionError, SessionShareError, DecryptionError,
+                     MatchingSessionDecryptionError, DeviceValidationError)
